@@ -34,10 +34,18 @@
 #include "soundlib/mod_specifications.h"
 #include "soundlib/AudioReadTarget.h"
 
+#if MPT_OS_WINDOWS && MPT_OS_WINDOWS_WINRT
+#include <windows.h>
+#endif // MPT_OS_WINDOWS && MPT_OS_WINDOWS_WINRT
+
 OPENMPT_NAMESPACE_BEGIN
 
 #if MPT_OS_WINDOWS && MPT_OS_WINDOWS_WINRT
-#if defined(_WIN32_WINNT)
+#if defined(NTDDI_VERSION)
+#if (NTDDI_VERSION < 0x06020000)
+MPT_WARNING("Warning: libopenmpt for WinRT is built with reduced functionality. Please #define NTDDI_VERSION 0x0602000.")
+#endif
+#elif defined(_WIN32_WINNT)
 #if (_WIN32_WINNT < 0x0602)
 #if MPT_COMPILER_MSVC
 #pragma message("Warning: libopenmpt for WinRT is built with reduced functionality. Please #define _WIN32_WINNT 0x0602.")
@@ -1079,6 +1087,7 @@ double module_impl::set_position_order_row( std::int32_t order, std::int32_t row
 	m_sndFile->m_PlayState.m_nCurrentOrder = static_cast<ORDERINDEX>( order );
 	m_sndFile->SetCurrentOrder( static_cast<ORDERINDEX>( order ) );
 	m_sndFile->m_PlayState.m_nNextRow = static_cast<ROWINDEX>( row );
+	m_sndFile->m_PlayState.m_nTickCount = CSoundFile::TICKS_ROW_FINISHED;
 	m_currentPositionSeconds = m_sndFile->GetLength( m_ctl_seek_sync_samples ? eAdjustSamplePositions : eAdjust, GetLengthTarget( static_cast<ORDERINDEX>( order ), static_cast<ROWINDEX>( row ) ) ).back().duration;
 	return m_currentPositionSeconds;
 }
