@@ -435,14 +435,14 @@ bool CSoundFile::ReadAMS(FileReader &file, ModLoadingFlags loadFlags)
 		file.ReadSizedString<uint8le, mpt::String::spacePadded>(ChnSettings[chn].szName);
 	}
 
-	// Read pattern names
+	// Read pattern names and create patterns
 	Patterns.ResizeArray(fileHeader.numPats);
 	for(PATTERNINDEX pat = 0; pat < fileHeader.numPats; pat++)
 	{
 		char name[11];
-		file.ReadSizedString<uint8le, mpt::String::spacePadded>(name);
+		const bool ok = file.ReadSizedString<uint8le, mpt::String::spacePadded>(name);
 		// Create pattern now, so name won't be reset later.
-		if(Patterns.Insert(pat, 64))
+		if(Patterns.Insert(pat, 64) && ok)
 		{
 			Patterns[pat].SetName(name);
 		}
@@ -969,8 +969,8 @@ bool CSoundFile::ReadAMS2(FileReader &file, ModLoadingFlags loadFlags)
 			}
 
 			char patternName[11];
-			patternChunk.ReadSizedString<uint8le, mpt::String::spacePadded>(patternName);
-			Patterns[pat].SetName(patternName);
+			if(patternChunk.ReadSizedString<uint8le, mpt::String::spacePadded>(patternName))
+				Patterns[pat].SetName(patternName);
 
 			ReadAMSPattern(Patterns[pat], true, patternChunk);
 		}
@@ -1030,27 +1030,27 @@ bool CSoundFile::ReadAMS2(FileReader &file, ModLoadingFlags loadFlags)
 /////////////////////////////////////////////////////////////////////
 // AMS Sample unpacking
 
-void AMSUnpack(const int8 * const source, size_t sourceSize, void * const dest, const size_t destSize, char packCharacter)
+void AMSUnpack(mpt::const_byte_span source, mpt::byte_span dest, int8 packCharacter)
 {
-	std::vector<int8> tempBuf(destSize, 0);
-	size_t depackSize = destSize;
+	std::vector<int8> tempBuf(dest.size(), 0);
+	std::size_t depackSize = dest.size();
 
 	// Unpack Loop
 	{
-		const int8 *in = source;
+		const mpt::byte *in = source.data();
 		int8 *out = tempBuf.data();
 
-		size_t i = sourceSize, j = destSize;
+		size_t i = source.size(), j = dest.size();
 		while(i != 0 && j != 0)
 		{
-			int8 ch = *(in++);
+			int8 ch = mpt::byte_cast<int8>(*(in++));
 			if(--i != 0 && ch == packCharacter)
 			{
-				uint8 repCount = *(in++);
+				uint8 repCount = mpt::byte_cast<uint8>(*(in++));
 				repCount = static_cast<uint8>(std::min(static_cast<size_t>(repCount), j));
 				if(--i != 0 && repCount)
 				{
-					ch = *(in++);
+					ch = mpt::byte_cast<int8>(*(in++));
 					i--;
 					while(repCount-- != 0)
 					{
@@ -1077,7 +1077,7 @@ void AMSUnpack(const int8 * const source, size_t sourceSize, void * const dest,
 		int8 *out = tempBuf.data();
 		uint16 bitcount = 0x80;
 		size_t k = 0;
-		uint8 *dst = static_cast<uint8 *>(dest);
+		uint8 *dst = mpt::byte_cast<uint8 *>(dest.data());
 		for(size_t i = 0; i < depackSize; i++)
 		{
 			uint8 al = *out++;
@@ -1088,7 +1088,7 @@ void AMSUnpack(const int8 * const source, size_t sourceSize, void * const dest,
 				bl = ((bl | (bl << 8)) >> ((dh + 8 - count) & 7)) & 0xFF;
 				bitcount = ((bitcount | (bitcount << 8)) >> 1) & 0xFF;
 				dst[k++] |= bl;
-				if(k >= destSize)
+				if(k >= dest.size())
 				{
 					k = 0;
 					dh++;
@@ -1101,16 +1101,16 @@ void AMSUnpack(const int8 * const source, size_t sourceSize, void * const dest,
 	// Delta Unpack
 	{
 		int8 old = 0;
-		int8 *out = static_cast<int8 *>(dest);
+		uint8 *out = mpt::byte_cast<uint8*>(dest.data());
 		for(size_t i = depackSize; i != 0; i--)
 		{
-			int pos = *reinterpret_cast<uint8 *>(out);
+			int pos = static_cast<uint8>(*out);
 			if(pos != 128 && (pos & 0x80) != 0)
 			{
 				pos = -(pos & 0x7F);
 			}
 			old -= static_cast<int8>(pos);
-			*(out++) = old;
+			*(out++) = static_cast<uint8>(old);
 		}
 	}
 }
