@@ -162,6 +162,10 @@ static void ConvertDBMEffect(uint8 &command, uint8 &param)
 			command = CMD_NONE;
 		break;
 
+	case CMD_PATTERNBREAK:
+		param = ((param >> 4) * 10) + (param & 0x0F);
+		break;
+
 #ifdef MODPLUG_TRACKER
 	case CMD_VIBRATO:
 		if(param & 0x0F)
@@ -427,7 +431,7 @@ bool CSoundFile::ReadDBM(FileReader &file, ModLoadingFlags loadFlags)
 			// Sample Info
 			mptSmp.Initialize();
 			mptSmp.nVolume = std::min<uint16>(instrHeader.volume, 64) * 4u;
-			mptSmp.nC5Speed = instrHeader.sampleRate;
+			mptSmp.nC5Speed = Util::muldivr(instrHeader.sampleRate, 8303, 8363);
 
 			if(instrHeader.loopLength && (instrHeader.flags & (DBMInstrument::smpLoop | DBMInstrument::smpPingPongLoop)))
 			{
