@@ -111,7 +111,7 @@ struct MO3Envelope
 	int16le points[25][2];
 
 	// Convert MO3 envelope data into OpenMPT's internal envelope format
-	void ConvertToMPT(InstrumentEnvelope &mptEnv, uint8 envShift) const
+	void ConvertToMPT(InstrumentEnvelope &mptEnv, uint8 envShift, MODTYPE type) const
 	{
 		if(flags & envEnabled) mptEnv.dwFlags.set(ENV_ENABLED);
 		if(flags & envSustain) mptEnv.dwFlags.set(ENV_SUSTAIN);
@@ -120,7 +120,7 @@ struct MO3Envelope
 		if(flags & envCarry) mptEnv.dwFlags.set(ENV_CARRY);
 		mptEnv.resize(std::min<uint8>(numNodes, 25));
 		mptEnv.nSustainStart = sustainStart;
-		mptEnv.nSustainEnd = sustainEnd;
+		mptEnv.nSustainEnd = (type == MOD_TYPE_XM) ? sustainStart : sustainEnd;
 		mptEnv.nLoopStart = loopStart;
 		mptEnv.nLoopEnd = loopEnd;
 		for(uint32 ev = 0; ev < mptEnv.size(); ev++)
@@ -190,9 +190,9 @@ struct MO3Instrument
 				mptIns.Keyboard[i] = sampleMap[i][1] + 1;
 			}
 		}
-		volEnv.ConvertToMPT(mptIns.VolEnv, 0);
-		panEnv.ConvertToMPT(mptIns.PanEnv, 0);
-		pitchEnv.ConvertToMPT(mptIns.PitchEnv, 5);
+		volEnv.ConvertToMPT(mptIns.VolEnv, 0, type);
+		panEnv.ConvertToMPT(mptIns.PanEnv, 0, type);
+		pitchEnv.ConvertToMPT(mptIns.PitchEnv, 5, type);
 		mptIns.nFadeOut = fadeOut;
 		if(midiChannel >= 128)
 		{
@@ -250,7 +250,7 @@ struct MO3Sample
 		smpCompressionMask	= 0x1000 | 0x2000 | 0x4000
 	};
 
-	int32le  freqFinetune;	// Frequency in S3M and IT, finetune (0...255) in MOD, MTM, XM
+	uint32le freqFinetune;	// Frequency in S3M and IT, finetune (0...255) in MOD, MTM, XM
 	int8le   transpose;
 	uint8le  defaultVolume;	// 0...64
 	uint16le panning;		// 0...256 if enabled, 0xFFFF otherwise
@@ -275,9 +275,9 @@ struct MO3Sample
 		if(type & (MOD_TYPE_IT | MOD_TYPE_S3M))
 		{
 			if(frequencyIsHertz)
-				mptSmp.nC5Speed = static_cast<uint32>(freqFinetune);
+				mptSmp.nC5Speed = freqFinetune;
 			else
-				mptSmp.nC5Speed = Util::Round<uint32>(8363.0 * std::pow(2.0, (freqFinetune + 1408) / 1536.0));
+				mptSmp.nC5Speed = Util::Round<uint32>(8363.0 * std::pow(2.0, static_cast<int32>(freqFinetune + 1408) / 1536.0));
 		} else
 		{
 			mptSmp.nFineTune = static_cast<int8>(freqFinetune);
@@ -356,7 +356,7 @@ struct MO3SampleChunk
 	strLen++; \
 	do { \
 		READ_CTRL_BIT; \
-		strLen = (strLen << 1) + carry; \
+		strLen = mpt::lshift_signed(strLen, 1) + carry; \
 		READ_CTRL_BIT; \
 	} while(carry); \
 }
@@ -402,7 +402,7 @@ static bool UnpackMO3Data(FileReader &file, uint8 *dst, uint32 size)
 			} else
 			{
 				// LZ ptr in ctrl stream
-				strOffset = (strLen << 8) | file.ReadUint8(); // read less significant offset byte from stream
+				strOffset = mpt::lshift_signed(strLen, 8) | file.ReadUint8(); // read less significant offset byte from stream
 				strLen = 0;
 				strOffset = ~strOffset;
 				if(strOffset < -1280)
@@ -415,9 +415,9 @@ static bool UnpackMO3Data(FileReader &file, uint8 *dst, uint32 size)
 
 			// read the next 2 bits as part of strLen
 			READ_CTRL_BIT;
-			strLen = (strLen << 1) + carry;
+			strLen = mpt::lshift_signed(strLen, 1) + carry;
 			READ_CTRL_BIT;
-			strLen = (strLen << 1) + carry;
+			strLen = mpt::lshift_signed(strLen, 1) + carry;
 			if(strLen == 0)
 			{
 				// length does not fit in 2 bits
@@ -884,7 +884,8 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 		}
 	}
 
-	ReadOrderFromFile<uint8>(Order(), musicChunk, fileHeader.numOrders, 0xFF, 0xFE);
+	const bool hasOrderSeparators = !(m_nType & (MOD_TYPE_MOD | MOD_TYPE_XM));
+	ReadOrderFromFile<uint8>(Order(), musicChunk, fileHeader.numOrders, hasOrderSeparators ? 0xFF : uint16_max, hasOrderSeparators ? 0xFE : uint16_max);
 
 	// Track assignments for all patterns
 	FileReader trackChunk = musicChunk.ReadChunk(fileHeader.numPatterns * fileHeader.numChannels * sizeof(uint16));
@@ -1269,15 +1270,15 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 		musicChunk.ReadNullString(name);
 		mpt::String::Copy(m_szNames[smp], name);
 		if(version >= 5)
-		{
 			musicChunk.ReadNullString(name);
-			mpt::String::Copy(sample.filename, name);
-		}
+		else
+			name.clear();
 
 		MO3Sample smpHeader;
 		if(!musicChunk.ReadStruct(smpHeader))
 			break;
 		smpHeader.ConvertToMPT(sample, m_nType, frequencyIsHertz);
+		mpt::String::Copy(sample.filename, name);
 
 		int16 sharedOggHeader = 0;
 		if(version >= 5 && (smpHeader.flags & MO3Sample::smpCompressionMask) == MO3Sample::smpSharedOgg)
@@ -1393,7 +1394,7 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 			// Note: Every Ogg stream has a unique serial number.
 			// stb_vorbis (currently) ignores this serial number so we can just stitch
 			// together our sample without adjusting the shared header's serial number.
-			const bool sharedHeader = sharedOggHeader != smp && sharedOggHeader > 0 && sharedOggHeader <= m_nSamples;
+			const bool sharedHeader = sharedOggHeader != smp && sharedOggHeader > 0 && sharedOggHeader <= m_nSamples && sampleChunk.headerSize > 0;
 
 #if defined(MPT_WITH_VORBIS) && defined(MPT_WITH_VORBISFILE)
 
