@@ -282,9 +282,15 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 		nonCompatTracker = true;
 		break;
 	case S3MFileHeader::trkOpenMPT:
-		trackerStr = MPT_USTRING("OpenMPT");
-		m_dwLastSavedWithVersion = (fileHeader.cwtv & S3MFileHeader::versionMask) << 16;
-		break; 
+		if(fileHeader.cwtv != S3MFileHeader::trkGraoumfTracker)
+		{
+			trackerStr = MPT_USTRING("OpenMPT");
+			m_dwLastSavedWithVersion = (fileHeader.cwtv & S3MFileHeader::versionMask) << 16;
+		} else
+		{
+			m_madeWithTracker = MPT_USTRING("Graoumf Tracker");
+		}
+		break;
 	case S3MFileHeader::trkBeRoTracker:
 		m_madeWithTracker = MPT_USTRING("BeRoTracker");
 		m_playBehaviour.set(kST3LimitPeriod);
@@ -292,6 +298,10 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 	case S3MFileHeader::trkCreamTracker:
 		m_madeWithTracker = MPT_USTRING("CreamTracker");
 		break;
+	default:
+		if(fileHeader.cwtv == S3MFileHeader::trkCamoto)
+			m_madeWithTracker = MPT_USTRING("Camoto");
+		break;
 	}
 	if(!trackerStr.empty())
 	{
@@ -567,6 +577,10 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 						else if(m.param > 0x08)
 							zxxCountRight++;
 					}
+				} else if(m.command == CMD_OFFSET && m.param == 0 && fileHeader.cwtv <= S3MFileHeader::trkST3_01)
+				{
+					// Offset command didn't have effect memory in ST3.01; fixed in ST3.03
+					m.command = CMD_NONE;
 				}
 			}
 		}
