@@ -27,7 +27,7 @@
 OPENMPT_NAMESPACE_BEGIN
 
 // Sample decompression routines in other source files
-void AMSUnpack(const int8 * const source, size_t sourceSize, void * const dest, const size_t destSize, char packCharacter);
+void AMSUnpack(mpt::const_byte_span source, mpt::byte_span dest, int8 packCharacter);
 uint8 MDLReadBits(uint32 &bitbuf, int32 &bitnum, const uint8 *(&ibuf), size_t &bytesLeft, int8 n);
 uintptr_t DMFUnpack(uint8 *psample, const uint8 *ibuf, const uint8 *ibufmax, uint32 maxlen);
 
@@ -57,6 +57,8 @@ size_t SampleIO::ReadSample(ModSample &sample, FileReader &file) const
 		restrictedSampleDataView = file.GetPinnedRawDataView(CalculateEncodedSize(sample.nLength));
 		sourceBuf = restrictedSampleDataView.data();
 		fileSize = restrictedSampleDataView.size();
+		if(sourceBuf == nullptr)
+			return 0;
 	} else
 	{
 		// Only DMF sample compression encoding should fall in this case,
@@ -171,7 +173,7 @@ size_t SampleIO::ReadSample(ModSample &sample, FileReader &file) const
 		LimitMax(sourceSize, mpt::saturate_cast<uint32>(packedDataView.size()));
 		bytesRead += sourceSize;
 
-		AMSUnpack(reinterpret_cast<const int8 *>(packedDataView.data()), packedDataView.size(), sample.pSample, sample.GetSampleSizeInBytes(), packCharacter);
+		AMSUnpack(packedDataView.span(), mpt::as_span(mpt::void_cast<mpt::byte*>(sample.pSample), sample.GetSampleSizeInBytes()), packCharacter);
 		if(sample.uFlags[CHN_16BIT] && !mpt::endian_is_little())
 		{
 			int16 *p = sample.pSample16;
