@@ -490,6 +490,8 @@ bool CSoundFile::Create(FileReader file, ModLoadingFlags loadFlags)
 	{
 		LimitMax(ChnSettings[ich].nVolume, uint16(64));
 		if (ChnSettings[ich].nPan > 256) ChnSettings[ich].nPan = 128;
+		if(ChnSettings[ich].nMixPlugin > MAX_MIXPLUGINS)
+			ChnSettings[ich].nMixPlugin = 0;
 		m_PlayState.Chn[ich].Reset(ModChannel::resetTotal, *this, ich);
 	}
 
@@ -553,6 +555,15 @@ bool CSoundFile::Create(FileReader file, ModLoadingFlags loadFlags)
 	// Set default play state values
 	if (!m_nDefaultTempo.GetInt()) m_nDefaultTempo.Set(125);
 	if (!m_nDefaultSpeed) m_nDefaultSpeed = 6;
+
+	if(m_nDefaultRowsPerMeasure < m_nDefaultRowsPerBeat)
+		m_nDefaultRowsPerMeasure = m_nDefaultRowsPerBeat;
+	LimitMax(m_nDefaultRowsPerBeat, MAX_ROWS_PER_BEAT);
+	LimitMax(m_nDefaultRowsPerMeasure, MAX_ROWS_PER_BEAT);
+	LimitMax(m_nDefaultGlobalVolume, MAX_GLOBAL_VOLUME);
+	if(!m_tempoSwing.empty())
+		m_tempoSwing.resize(m_nDefaultRowsPerBeat);
+
 	m_PlayState.m_nMusicSpeed = m_nDefaultSpeed;
 	m_PlayState.m_nMusicTempo = m_nDefaultTempo;
 	m_PlayState.m_nCurrentRowsPerBeat = m_nDefaultRowsPerBeat;
@@ -567,7 +578,7 @@ bool CSoundFile::Create(FileReader file, ModLoadingFlags loadFlags)
 	m_PlayState.m_nPattern = 0;
 	m_PlayState.m_nBufferCount = 0;
 	m_PlayState.m_dBufferDiff = 0;
-	m_PlayState.m_nTickCount = m_PlayState.m_nMusicSpeed;
+	m_PlayState.m_nTickCount = TICKS_ROW_FINISHED;
 	m_PlayState.m_nNextRow = 0;
 	m_PlayState.m_nRow = 0;
 	m_PlayState.m_nPatternDelay = 0;
@@ -797,7 +808,7 @@ void CSoundFile::ResetPlayPos()
 
 	m_PlayState.m_nNextOrder = 0;
 	m_PlayState.m_nNextRow = 0;
-	m_PlayState.m_nTickCount = m_PlayState.m_nMusicSpeed;
+	m_PlayState.m_nTickCount = TICKS_ROW_FINISHED;
 	m_PlayState.m_nBufferCount = 0;
 	m_PlayState.m_nPatternDelay = 0;
 	m_PlayState.m_nFrameDelay = 0;
@@ -846,7 +857,7 @@ void CSoundFile::SetCurrentOrder(ORDERINDEX nOrder)
 		m_PlayState.m_nNextOrder = nOrder;
 		m_PlayState.m_nRow = m_PlayState.m_nNextRow = 0;
 		m_PlayState.m_nPattern = 0;
-		m_PlayState.m_nTickCount = m_PlayState.m_nMusicSpeed;
+		m_PlayState.m_nTickCount = TICKS_ROW_FINISHED;
 		m_PlayState.m_nBufferCount = 0;
 		m_PlayState.m_nPatternDelay = 0;
 		m_PlayState.m_nFrameDelay = 0;
@@ -971,7 +982,7 @@ void CSoundFile::LoopPattern(PATTERNINDEX nPat, ROWINDEX nRow)
 		if(nRow >= Patterns[nPat].GetNumRows()) nRow = 0;
 		m_PlayState.m_nPattern = nPat;
 		m_PlayState.m_nRow = m_PlayState.m_nNextRow = nRow;
-		m_PlayState.m_nTickCount = m_PlayState.m_nMusicSpeed;
+		m_PlayState.m_nTickCount = TICKS_ROW_FINISHED;
 		m_PlayState.m_nPatternDelay = 0;
 		m_PlayState.m_nFrameDelay = 0;
 		m_PlayState.m_nNextPatStartRow = 0;
@@ -987,7 +998,7 @@ void CSoundFile::DontLoopPattern(PATTERNINDEX nPat, ROWINDEX nRow)
 	if(nRow >= Patterns[nPat].GetNumRows()) nRow = 0;
 	m_PlayState.m_nPattern = nPat;
 	m_PlayState.m_nRow = m_PlayState.m_nNextRow = nRow;
-	m_PlayState.m_nTickCount = m_PlayState.m_nMusicSpeed;
+	m_PlayState.m_nTickCount = TICKS_ROW_FINISHED;
 	m_PlayState.m_nPatternDelay = 0;
 	m_PlayState.m_nFrameDelay = 0;
 	m_PlayState.m_nBufferCount = 0;
