@@ -1,7 +1,8 @@
 /*
  * UpdateModule.cpp
  * ----------------
- * Purpose: CSoundFile functions for correcting modules made with previous versions of OpenMPT.
+ * Purpose: Compensate for playback bugs of previous OpenMPT versions during import
+ *          by rewriting patterns / samples / instruments or enabling / disabling specific compatibility flags
  * Notes  : (currently none)
  * Authors: OpenMPT Devs
  * The OpenMPT source code is released under the BSD license. Read LICENSE for more details.
@@ -10,6 +11,7 @@
 
 #include "stdafx.h"
 #include "Sndfile.h"
+#include "plugins/PluginManager.h"
 #include "../common/StringFixer.h"
 #include "../common/version.h"
 
@@ -113,6 +115,12 @@ struct UpgradePatternData
 					m.vol = 0;
 				}
 			}
+
+			// Command I11 accidentally behaved the same as command I00 with compatible IT tremor and old effects disabled
+			if(m.command == CMD_TREMOR && m.param == 0x11 && version < MAKE_VERSION_NUMERIC(1, 29, 12, 02) && sndFile.m_playBehaviour[kITTremor] && !sndFile.m_SongFlags[SONG_ITOLDEFFECTS])
+			{
+				m.param = 0;
+			}
 		}
 
 		else if(modType == MOD_TYPE_XM)
@@ -251,7 +259,7 @@ void CSoundFile::UpgradeModule()
 
 			if(!compatModeIT || m_dwLastSavedWithVersion < MAKE_VERSION_NUMERIC(1, 18, 00, 00))
 			{
-				// Previously, Pitch/Pan Separation was only half depth.
+				// Previously, Pitch/Pan Separation was only half depth (plot twist: it was actually only quarter depth).
 				// This was corrected in compatible mode in OpenMPT 1.18, and in OpenMPT 1.20 it is corrected in normal mode as well.
 				ins->nPPS = (ins->nPPS + (ins->nPPS >= 0 ? 1 : -1)) / 2;
 			}
@@ -600,6 +608,24 @@ void CSoundFile::UpgradeModule()
 		// Frequency slides were always in Hz rather than periods in this version range.
 		m_playBehaviour.set(kHertzInLinearMode);
 	}
+
+	if(m_dwLastSavedWithVersion >= MAKE_VERSION_NUMERIC(1, 27, 00, 00) && m_dwLastSavedWithVersion < MAKE_VERSION_NUMERIC(1, 30, 06, 00) && (GetType() & (MOD_TYPE_IT | MOD_TYPE_MPT | MOD_TYPE_XM)))
+	{
+		// Fix off-by-one delay length in older Echo DMO emulation
+		for(auto &plugin : m_MixPlugins)
+		{
+			if(plugin.Info.dwPluginId1 == kDmoMagic && plugin.Info.dwPluginId2 == int32(0xEF3E932C) && plugin.pluginData.size() == 24)
+			{
+				float32le leftDelay, rightDelay;
+				memcpy(&leftDelay, plugin.pluginData.data() + 12, 4);
+				memcpy(&rightDelay, plugin.pluginData.data() + 16, 4);
+				leftDelay = float32le{mpt::safe_clamp(((leftDelay * 2000.0f) - 1.0f) / 1999.0f, 0.0f, 1.0f)};
+				rightDelay = float32le{mpt::safe_clamp(((rightDelay * 2000.0f) - 1.0f) / 1999.0f, 0.0f, 1.0f)};
+				memcpy(plugin.pluginData.data() + 12, &leftDelay, 4);
+				memcpy(plugin.pluginData.data() + 16, &rightDelay, 4);
+			}
+		}
+	}
 }
 
 
